import { RegularTable, TreeHeader } from "./util";
// await customElements.whenDefined('regular-table');
const RegularTableElement = customElements.get('regular-table');
const DATE_FORMATTER = new Intl.DateTimeFormat("en-us");
export class TreeFinderGridElement extends RegularTableElement {
    constructor() {
        super(...arguments);
        this._initializedListeners = false;
    }
    async init(model, options = {}) {
        const { doWindowReize = false, } = options;
        this.options = {
            doWindowReize,
        };
        this.model = model;
        this.setDataListener((x0, y0, x1, y1) => this.dataListener(x0, y0, x1, y1));
        // run listener initializations only once
        if (!this._initializedListeners) {
            this.addStyleListener(() => this.columnHeaderStyleListener());
            this.addStyleListener(() => this.rowStyleListener());
            this.addEventListener("mousedown", event => this.onSortClick(event));
            this.addEventListener("mousedown", event => this.onTreeClick(event));
            this.addEventListener("dblclick", event => this.onRowDoubleClick(event));
            // this.addEventListener("scroll", () => (this as any)._resetAutoSize());
            // click debug listener
            // this.addEventListener("mousedown", event => RegularTable.clickLoggingListener(event, this));
            if (this.options.doWindowReize) {
                // resize whenever window size changes, if requested
                window.addEventListener('resize', async () => {
                    await this.draw();
                });
            }
            this._initializedListeners = true;
        }
        await this.draw();
    }
    async dataListener(start_col, start_row, end_col, end_row) {
        const data = [];
        for (let cix = start_col; cix < end_col - 1; cix++) {
            const column = this.model.columns[cix];
            data.push(this.model.contents.slice(start_row, end_row).map(content => {
                const val = content.row[column];
                return val instanceof Date ? DATE_FORMATTER.format(val) : val;
            }));
        }
        return {
            // num_columns/rows: number -> count of cols/rows
            num_columns: this.model.columns.length,
            num_rows: this.model.contents.length,
            // column/row_headers: string[] -> arrays of path parts that get displayed as the first value in each col/row. Length > 1 implies a tree structure
            column_headers: this.model.columns.map(col => [col]),
            row_headers: this.model.contents.slice(start_row, end_row).map(x => {
                return [TreeHeader.treeHeader({
                        isDir: x.isDir,
                        isOpen: x.isOpen,
                        path: x.getPathAtDepth(this.model.pathDepth),
                    })];
            }),
            // data: object[][] -> array of arrays, each subarray containing all of the visible values for one column
            data,
        };
    }
    columnHeaderStyleListener() {
        var _a;
        for (const th of this.querySelectorAll("thead tr:last-child th")) {
            const { column_header, x } = this.getMeta(th);
            const columnName = column_header[column_header.length - 1];
            if (columnName) {
                const sortOrder = (_a = this.model.sortStates.byColumn[columnName === "0" ? "path" : columnName]) === null || _a === void 0 ? void 0 : _a.order;
                th.classList.toggle(`tf-header-sort-${sortOrder}`, !!sortOrder);
            }
            th.classList.toggle("tf-header-corner", typeof x === "undefined");
            th.classList.toggle("tf-header", true);
            th.classList.toggle("tf-header-align-left", true);
        }
    }
    rowStyleListener() {
        const spans = this.querySelectorAll("tbody th .rt-group-name");
        for (const span of spans) {
            // style the browser's filetype icons
            const { y, value } = RegularTable.metadataFromElement(span, this);
            if (value) {
                const kind = this.model.contents[y].row.kind;
                span.classList.add("tf-browser-filetype-icon", `tf-browser-${kind}-icon`);
            }
        }
    }
    onSortClick(event) {
        if (event.button !== 0) {
            return;
        }
        const element = event.target;
        if (element.classList.contains("rt-column-resize")) {
            // don't sort when the column resize handle nodes are clicked
            return;
        }
        const metadata = RegularTable.metadataFromElement(element, this);
        if (!metadata || !RegularTable.columnHeaderClicked(metadata)) {
            return;
        }
        this.model.sort({
            col: metadata.value || this.model.sortStates.defaultColumn,
            multisort: event.shiftKey,
        });
        this.draw();
    }
    onTreeClick(event) {
        if (event.button !== 0) {
            return;
        }
        let element = event.target;
        if (!element.classList.contains("rt-row-header-icon")) {
            // only open/close node when open/close icon is clicked
            return;
        }
        // assert that metadata exists, given element.classList check above
        const metadata = RegularTable.metadataFromElement(element, this);
        if (this.model.contents[metadata.y].isOpen) {
            this.model.collapse(metadata.y);
        }
        else {
            this.model.expand(metadata.y);
        }
        this._resetAutoSize();
        this.draw();
    }
    onRowDoubleClick(event) {
        if (event.button !== 0) {
            return;
        }
        const element = event.target;
        const metadata = RegularTable.metadataFromElement(element, this);
        if (!metadata || !RegularTable.rowClicked(metadata)) {
            return;
        }
        // event.preventDefault();
        // event.stopPropagation();
        // event.stopImmediatePropagation();
        // event.returnValue = false;
        const newRootContent = this.model.contents[metadata.y];
        if (newRootContent.isDir) {
            this.model.setRoot(newRootContent.row);
            // .init() calls .draw()
            this.init(this.model, this.options);
        }
    }
}
//# sourceMappingURL=grid.js.map